<?php

namespace elggBook\elggBookTools;
	
use ElggMenuItem;
use ElggRiverItem;
use UFCOE\Elgg\MenuList;
use ElggEntity;
use ElggFile;
use ElggGroup;
use ElggObject;
use ElggUser;

class Menus {
	
	/**THEWIRE TOOLS
	 * Add reshare menu items to the entity menu
	 *
	 * @param string          $hook        the name of the hook
	 * @param string          $type        the type of the hook
	 * @param \ElggMenuItem[] $returnvalue current return value
	 * @param array           $params      supplied params
	 *
	 * @return void|\ElggMenuItem[]
	 */
	public static function entityRegisterReshare($hook, $type, $returnvalue, $params) {
		
		if (!elgg_is_logged_in()) {
			return;
		}
		
		$entity = elgg_extract('entity', $params);
		if (!($entity instanceof \ElggEntity)) {
			return;
		}
		
		if (!self::canReshareEntity($entity)) {
			return;
		}
		
		elgg_load_js('elgg.thewire');
	
		elgg_load_js('lightbox');
		elgg_load_css('lightbox');
	
		$reshare_guid = $entity->getGUID();
		$reshare = null;
		
		if (elgg_instanceof($entity, 'object', 'thewire')) {
			$reshare = $entity->getEntitiesFromRelationship([
				'relationship' => 'reshare',
				'limit' => 1,
				'callback' => function($row) {
					return (int) $row->guid;
				},
			]);
			if ($reshare) {
				// this is a wire post which is a reshare, so link to original object
				$reshare_guid = $reshare[0];
			}
		}
		
		if (empty($reshare)) {
			// check is this item was shared on thewire
			$count = $entity->getEntitiesFromRelationship([
				'type' => 'object',
				'subtype' => 'thewire',
				'relationship' => 'reshare',
				'inverse_relationship' => true,
				'count' => true,
			]);
			
			if ($count) {
				// show counter
				$returnvalue[] = \ElggMenuItem::factory([
					'name' => 'thewire_tools_reshare_count',
					'text' => $count,
					'title' => elgg_echo('thewire_tools:reshare:count'),
					'href' => 'ajax/view/thewire_tools/reshare_list?entity_guid=' . $reshare_guid,
					'link_class' => 'elgg-lightbox',
					'is_trusted' => true,
					'priority' => 501,
					'data-colorbox-opts' => json_encode(['maxHeight' => '85%']),
				]);
			}
		}
	
		$returnvalue[] = \ElggMenuItem::factory([
			'name' => 'thewire_tools_reshare',
			'text' => '<i class="fa  fa-microphone fa-lg"></i>',
			'title' => elgg_echo('thewire_tools:reshare'),
			'href' => 'ajax/view/thewire_tools/reshare?reshare_guid=' . $reshare_guid,
			'link_class' => 'elgg-lightbox',
			'is_trusted' => true,
			'priority' => 500,
		]);
		
		return $returnvalue;
	}
	
	/**
	 * Check if resharing of this entity is allowed
	 *
	 * @param \ElggEntity $entity the entity to check
	 *
	 * @return bool
	 */
	protected static function canReshareEntity(\ElggEntity $entity) {
		
		if (!($entity instanceof \ElggEntity)) {
			return false;
		}
		
		// only allow objects and groups
		if (!($entity instanceof \ElggObject) && !($entity instanceof \ElggGroup)) {
			return false;
		}
		
		// comments and discussion replies are never allowed
		$blocked_subtypes = ['comment', 'discussion_reply'];
		if (in_array($entity->getSubtype(), $blocked_subtypes)) {
			return false;
		}
		
		// by default allow searchable entities
		$reshare_allowed = false;
		if ($entity instanceof \ElggGroup) {
			$reshare_allowed = true;
		} else {
			$searchable_entities = get_registered_entity_types($entity->getType());
			if (!empty($searchable_entities)) {
				$reshare_allowed = in_array($entity->getSubtype(), $searchable_entities);
			}
		}
		
		// trigger hook to allow others to change
		$params = [
			'entity' => $entity,
			'user' => elgg_get_logged_in_user_entity(),
		];
		return (bool) elgg_trigger_plugin_hook('reshare', $entity->getType(), $params, $reshare_allowed);
	}

	/**HYPEDISCOVERY
	 * Setup entity menu
	 *
	 * @param string         $hook   "register"
	 * @param string         $type   "menu:extras"
	 * @param ElggMenuItem[] $return Menu
	 * @param array          $params Hook params
	 * @return ElggMenuItem[]
	 */
	public static function extrasMenuSetup($hook, $type, $return, $params) {

		$segments = _elgg_services()->request->getUrlSegments();
		$share_url = elgg_normalize_url(implode('/', $segments));
		$entity = get_entity_from_url($share_url);
		$guid = $entity->guid;
		if ($entity instanceof ElggSite) {
			$guid = '';
		}

		$providers = get_discovery_providers();
		if (empty($providers)) {
			return;
		}

		$text = '<i class="fa  fa-share-alt fa-lg" style="color:#27a2a7"></i>' . elgg_echo('Social shares');
		$return[] = ElggMenuItem::factory(array(
					'name' => 'discovery:share',
					'text' => $text,
					'href' => elgg_http_add_url_query_elements("opengraph/share/$guid", [
						'share_url' => $share_url,
					]),
					'title' => elgg_echo('discovery:entity:share'),
					'link_class' => 'elgg-lightbox',
					'data-colorbox-opts' => json_encode([
						'maxWidth' => '600px',
					]),
					'data' => [
						'icon' => 'share',
					],
					'priority' => 700,
					'deps' => ['elgg/lightbox'],
		));

		return $return;
	}
	
	/**HYPEICONS
	 * Setup user hover menu
	 *
	 * @param string         $hook   "register"
	 * @param string         $type   "menu:user_hover"
	 * @param ElggMenuItem[] $return Menu
	 * @param array          $params Hook params
	 * @return ElggMenuItem[]
	 */
	public static function setupUserHoverMenu($hook, $type, $return, $params) {

		$entity = elgg_extract('entity', $params);
		if (!$entity instanceof ElggUser) {
			return;
		}

		if ($edit_icon = self::prepareEditIconItem($entity)) {
			$edit_icon->setText($edit_icon->getTooltip());
			//$section = elgg_is_admin_logged_in() ? 'admin' : 'actions';
			//$edit_icon->setSection($section);
			$return[] = $edit_icon;
		}

		if ($edit_cover = self::prepareEditCoverItem($entity)) {
			$edit_cover->setText($edit_cover->getTooltip());
			//$section = elgg_is_admin_logged_in() ? 'admin' : 'actions';
			//$edit_cover->setSection($section);
			$return[] = $edit_cover;
		}

		return $return;
	}
}
